use lazy_static::lazy_static;
use pic8259::ChainedPics;
use spin::Mutex;
use x86_64::{
	instructions::{interrupts, port::Port},
	structures::idt::{InterruptDescriptorTable, InterruptStackFrame},
};

use crate::{hard::keyboard::keyboard, println};

pub fn init() {
	println!("Initializing interrupt handler.");
	IDT.load();
	unsafe { PICS.lock().initialize() };
	interrupts::enable();
}

const INT_OFFSET: u8 = 32;
const INT_IDX_TIMER: u8 = 32;
const INT_IDX_KEYBOARD: u8 = 33;

lazy_static! {
	static ref IDT: InterruptDescriptorTable = {
		let mut idt = InterruptDescriptorTable::new();
		let entries = idt.slice_mut(INT_OFFSET..);
		entries[(INT_IDX_TIMER - INT_OFFSET) as usize].set_handler_fn(timer_handler);
		entries[(INT_IDX_KEYBOARD - INT_OFFSET) as usize].set_handler_fn(keyboard_handler);
		idt
	};
}

static PICS: Mutex<ChainedPics> = Mutex::new(unsafe { ChainedPics::new(0x20, 0x28) });

extern "x86-interrupt" fn keyboard_handler(_stack_frame: InterruptStackFrame) {
	let mut port = Port::new(0x60);
	let scan_code = unsafe { port.read() };
	keyboard().record(scan_code);
	unsafe { PICS.lock().notify_end_of_interrupt(INT_IDX_KEYBOARD) };
}

extern "x86-interrupt" fn timer_handler(_stack_frame: InterruptStackFrame) {
	// print!(".");
	unsafe { PICS.lock().notify_end_of_interrupt(INT_IDX_TIMER) };
}
