use x86_64::instructions::{interrupts, port::Port};

pub const WIDTH: usize = 80;
pub const HEIGHT: usize = 25;

pub struct Vga;

impl Vga {
	const BUFF: *mut u8 = 0xb8000 as _;
	pub fn write_at(&self, pos: (usize, usize), text: char) {
		let index = (pos.0 + pos.1 * WIDTH) as isize;
		let byte = text as u8;
		unsafe {
			*Vga::BUFF.offset(index * 2) = byte;
			*Vga::BUFF.offset(index * 2 + 1) = 0xb;
		}
	}

	pub fn set_cursor_pos(&self, pos: (usize, usize)) {
		type Uk = u16;
		let index = (pos.0 + pos.1 * WIDTH) as Uk + 1;
		interrupts::without_interrupts(|| unsafe {
			let mut cmd = Port::<Uk>::new(0x3D4);
			let mut data = Port::<Uk>::new(0x3D5);
			cmd.write(0x0F);
			data.write(index & 0xFF);

			cmd.write(0x0E);
			data.write((index >> 8) & 0xFF);
		});
	}
}

#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(u8)]
pub enum Color {
	Black = 0,
	Blue = 1,
	Green = 2,
	Cyan = 3,
	Red = 4,
	Magenta = 5,
	Brown = 6,
	LightGray = 7,
	DarkGray = 8,
	LightBlue = 9,
	LightGreen = 10,
	LightCyan = 11,
	LightRed = 12,
	Pink = 13,
	Yellow = 14,
	White = 15,
}

#[derive(Debug, Clone, Copy, PartialEq, Eq)]
#[repr(transparent)]
struct ColorCode(u8);

impl ColorCode {
	fn new(foreground: Color, background: Color) -> ColorCode {
		ColorCode((background as u8) << 4 | (foreground as u8))
	}
}
