use core::mem;

#[allow(clippy::empty_loop)]
pub fn spin() -> ! {
	loop {}
}

#[derive(Debug, Clone)]
pub struct ArrayQueue<T, const N: usize> {
	inner: [Option<T>; N],
}

impl<T, const N: usize> ArrayQueue<T, N> {
	pub fn new() -> Self {
		let inner = [const { None }; N];
		Self { inner }
	}

	pub fn add(&mut self, item: T) -> Option<T> {
		let mut current = Some(item);
		for i in 0..N {
			mem::swap(&mut current, &mut self.inner[i]);
			if current.is_none() {
				break;
			}
		}
		current
	}

	pub fn content(&self) -> &[Option<T>; N] {
		&self.inner
	}

	pub fn flush(&mut self) -> [Option<T>; N] {
		let other = [const { None }; N];
		mem::replace(&mut self.inner, other)
	}
}

impl<T, const N: usize> Default for ArrayQueue<T, N> {
	fn default() -> Self {
		Self::new()
	}
}
